-- local is_in_beta = GameIsBetaBuild()

function replace_value(path, input, output, print_bool)
	local content = ModTextFileGetContent(path)
	content = content:gsub(input, output)
	ModTextFileSetContent(path, content)
	if print_bool then
		print(content)
	end
end

local function print_tabbed(tabbing,text)
	msg = ""
	for i = 1,tabbing do
		msg = msg.."\t"
	end
	print(msg..text)
end
function print_any(object,iter,prefix)
	iter = iter or 0
	prefix = prefix or ""
	if type(object) == "table" then
		if prefix == "" then
			print_tabbed(iter,prefix.."{")
		else
			print_tabbed(iter,prefix..": {")
		end
		for k,v in pairs(object) do
			print_any(v,iter+1,tostring(k))
		end
		print_tabbed(iter,"}")
	else
		if prefix == "" then
			print_tabbed(iter,type(object):sub(1,2):upper().."("..tostring(object)..")")
		else
			print_tabbed(iter,table.concat({prefix,": ",type(object):sub(1,3),"(",tostring(object),")"}))
		end
	end
end

setting_cache = setting_cache or {}
local function get_setting(name)
    setting_cache[name] = setting_cache[name] or ModSettingGet(name)
    return setting_cache[name]
end

local nxml = dofile_once("mods/inverse_gravity/nxml.lua")

-- Load and parse the XML content
local content = ModTextFileGetContent("data/materials.xml")
local xml = nxml.parse(content)
local xml_orig = nxml.parse(content)

-- ModMaterialsFileAdd("mods/inverse_gravity/materials.xml")

function create_unique_wang_color_string_generator()
  local start = 0xB426C528 + 1000
  local counter = 0
  return function()
    counter = counter + 1
    return ("%8X"):format(start + counter)
  end
end
local create_unique_wang = create_unique_wang_color_string_generator()


local lookup_by_name = {}
for element in xml:each_child() do
  if element.attr.name then
    lookup_by_name[element.attr.name] = element
  end
end

local lookup_by_name_orig = {}
for element in xml_orig:each_child() do
  if element.attr.name then
    lookup_by_name_orig[element.attr.name] = element
  end
end

--Add reactions for material identifier sands
-- ModMaterialsFileAdd("mods/inverse_gravity/materials.xml")
if get_setting("inverse_gravity.fire_gravity") then
	ModMagicNumbersFileAdd( "mods/inverse_gravity/fire_magic.xml" ) 
end

--Increasing material Limit
ModMagicNumbersFileAdd( "mods/inverse_gravity/increase_material_limit.xml" ) 

local mode = ModSettingGet( "inverse_gravity.physics_gravity" )

local sand_count = 0

-- Iterate through each element in the XML
for element in xml:each_child() do

	function string_split(inputstr, sep)
	  sep = sep or "%s"
	  local t = {}
	  for str in string.gmatch(inputstr, "([^"..sep.."]+)") do
		table.insert(t, str)
	  end
	  return t
	end

	function has_tag(element, tag_to_check)
	  local tags = string_split(element.attr.tags or "", ",")
	  for i,tag in ipairs(tags) do
		if tag == tag_to_check then
		  return true
		end
	  end
	  return false
	end

	if mode == "normal" then
	elseif mode == "inverted" then
		element.attr.solid_gravity_scale = (element.attr.solid_gravity_scale or 1) * -1
	elseif mode == "zero" then
		element.attr.solid_gravity_scale = (element.attr.solid_gravity_scale or 1) * 0
	end
	
	-- print(tostring(element.attr.gas_horizontal_speed))
	-- print(tostring(element.attr.gas_upwards_speed))
	-- if element.attr.name == "acid_gas" then
		-- print_any(element)
	-- end
	
	excluded_materials = {}
	
	all_criteria_met = false
	
	if get_setting("inverse_gravity.sand_gravity") then
		
		if ( element.attr.cell_type == "liquid" ) and 
			( element.attr.liquid_static ~= "1") and 
			( element.attr.liquid_sand == "1") and
			( element.attr.name ~= "soil") and
			( element.attr.name ~= "soil_lush") and
			( element.attr.name ~= "moss") and
			( element.attr.name ~= "grass") and
			( element.attr.name ~= "gold") and
			( element.attr.name ~= "gold_radioactive") and
			( element.attr.name ~= "coal_static") and
			-- ( element.attr.name ~= "poo") and
			( element.attr.name ~= "sandstone") and
			( element.attr.name ~= "endslime_blood") and
			( element.attr.name ~= "snow_sticky") and
			( has_tag(element, "[sand_ground]") or has_tag(element, "[sand_metal]") or has_tag(element, "[sand_other]") or has_tag(element, "[plant]")) then
			all_criteria_met = true
			if element.attr.tags then
			  element.attr.tags = element.attr.tags .. ",[material_identifier]"
			elseif not element.attr.tags then
				if parent_element.attr.tags then
					element.attr.tags = parent_element.attr.tags .. ",[material_identifier]"
					-- print("Yes 2 baby")
				end
			else
			  element.attr.tags = "[material_identifier]"
			end
			-- if element.attr.name then
				-- print("Yes 1 baby ".. tostring(element.attr.name) .. tostring(element.attr.tags))
			-- end
		elseif element.attr._parent then
			parent_element = lookup_by_name[element.attr._parent]
			-- print("Yes Parent Found baby ".. tostring(element.attr.name) .. tostring(element.attr._parent))
			-- print("Parent Values are ".. tostring(parent_element.attr.cell_type) .. tostring(parent_element.attr.liquid_static) .. tostring(parent_element.attr.liquid_sand))
			if ( parent_element.attr.cell_type == "liquid" ) and 
			( parent_element.attr.liquid_static ~= 1) and 
			( parent_element.attr.liquid_sand == 1) and 
			( element.attr.name ~= "soil") and
			( element.attr.name ~= "soil_lush") and
			( element.attr.name ~= "moss") and
			( element.attr.name ~= "grass") and
			( element.attr.name ~= "gold") and
			( element.attr.name ~= "gold_radioactive") and
			-- ( element.attr.name ~= "honey") and
			-- ( element.attr.name ~= "poo") and
			( element.attr.name ~= "sandstone") and
			( element.attr.name ~= "endslime_blood") and
			( element.attr.name ~= "snow_sticky") and
			( has_tag(parent_element, "[sand_ground]") or has_tag(parent_element, "[sand_metal]") or has_tag(parent_element, "[sand_other]") or has_tag(parent_element, "[plant]") ) then
			
				
				
				if element.attr.tags then
				  element.attr.tags = element.attr.tags .. ",[material_identifier]"
				  -- if element.attr.name then
					-- print("BINGO EXTRA 1")
				  -- end
				elseif not element.attr.tags then
					-- if element.attr.name then
						-- print("BINGO EXTRA 2")
					-- end
					if parent_element.attr.tags then
						element.attr.tags = parent_element.attr.tags .. ",[material_identifier]"
						-- if element.attr.name then
							-- print("BINGO EXTRA 3")
						-- end
					end
				else
				  element.attr.tags = "[material_identifier]"
				  -- if element.attr.name then
					-- print("BINGO EXTRA 4")
				  -- end
				end
				all_criteria_met = true
				-- if element.attr.name then
					-- print("Yes 2 baby ".. tostring(element.attr.name)  .. tostring(element.attr.tags))
				-- end
			end
		else
			all_criteria_met = false
			if element.attr.name then
				-- print("No Sorry baby ".. tostring(element.attr.name) .. tostring(element.attr.tags))
			end
		end
	end
	
	if ( all_criteria_met == true ) then
	
		if get_setting("inverse_gravity.sand_gravity") then

			element.attr.cell_type = "liquid"
			element.attr.liquid_static = 0
			element.attr.liquid_gravity = -5
			element.attr.liquid_sand = 1
			element.attr.liquid_sticks_to_ceiling=5000
			element.attr.stickyness=5000
			
			sand_count = sand_count + 1
			material = nxml.parse(([[<CellDataChild tags="[nil]" cell_type="gas" platform_type="0" liquid_sand="0" liquid_static="0" gas_speed="250" gas_horizontal_speed="40" _parent="%s" _inherit_reactions="0" name="%s_inverse_gravity_gas_form" wang_color="%s" convert_to_box2d_material="%s" ignore_self_reaction_warning="1"></CellDataChild>]]):format(element.attr.name,element.attr.name,create_unique_wang(),element.attr.name))
			
			xml:add_child(material)
			reaction = nxml.parse(([[<Reaction probability="100" input_cell1="%s_inverse_gravity_gas_form" input_cell2="[nil]" output_cell1="%s" output_cell2="[nil]"></Reaction>]]):format(element.attr.name,element.attr.name))
			xml:add_child(reaction)
			reaction = nxml.parse(([[<Reaction probability="100" input_cell1="%s_inverse_gravity_gas_form" input_cell2="[solid]" output_cell1="%s" output_cell2="[solid]"></Reaction>]]):format(element.attr.name,element.attr.name))
			xml:add_child(reaction)
			reaction = nxml.parse(([[<Reaction probability="100" input_cell1="%s_inverse_gravity_gas_form" input_cell2="[box2d]" output_cell1="%s" output_cell2="[box2d]"></Reaction>]]):format(element.attr.name,element.attr.name))
			xml:add_child(reaction)
			reaction = nxml.parse(([[<Reaction probability="100" input_cell1="%s_inverse_gravity_gas_form" input_cell2="[static]" output_cell1="%s" output_cell2="[static]"></Reaction>]]):format(element.attr.name,element.attr.name))
			xml:add_child(reaction)
			reaction = nxml.parse(([[<Reaction probability="100" input_cell1="%s_inverse_gravity_gas_form" input_cell2="[sand_ground]" output_cell1="%s" output_cell2="[sand_ground]"></Reaction>]]):format(element.attr.name,element.attr.name))
			xml:add_child(reaction)
			reaction = nxml.parse(([[<Reaction probability="100" input_cell1="%s_inverse_gravity_gas_form" input_cell2="[sand_metal]" output_cell1="%s" output_cell2="[sand_metal]"></Reaction>]]):format(element.attr.name,element.attr.name))
			xml:add_child(reaction)
			reaction = nxml.parse(([[<Reaction probability="100" input_cell1="%s_inverse_gravity_gas_form" input_cell2="[sand_other]" output_cell1="%s" output_cell2="[sand_other]"></Reaction>]]):format(element.attr.name,element.attr.name))
			xml:add_child(reaction)
			reaction = nxml.parse(([[<Reaction probability="100" input_cell1="%s_inverse_gravity_gas_form" input_cell2="[plant]" output_cell1="%s" output_cell2="[plant]"></Reaction>]]):format(element.attr.name,element.attr.name))
			xml:add_child(reaction)
			-- reaction = nxml.parse(([[<Reaction probability="100" input_cell1="[custom_id%s]" input_cell2="air" output_cell1="air" output_cell2="%s_inverse_gravity_gas_form" direction="top"></Reaction>]]):format(element.attr.name,element.attr.name))
			-- xml:add_child(reaction)
			-- reaction = nxml.parse(([[<Reaction probability="100" input_cell1="%s" input_cell2="[liquid]" output_cell1="[liquid]" output_cell2="%s" direction="top"></Reaction>]]):format(element.attr.name,element.attr.name))
			-- xml:add_child(reaction)
			-- reaction = nxml.parse(([[<Reaction probability="100" input_cell1="%s" input_cell2="[sand_ground]" output_cell1="[sand_ground]" output_cell2="%s" direction="top"></Reaction>]]):format(element.attr.name,element.attr.name))
			-- xml:add_child(reaction)
			-- reaction = nxml.parse(([[<Reaction probability="100" input_cell1="%s" input_cell2="[sand_metal]" output_cell1="[sand_metal]" output_cell2="%s" direction="top"></Reaction>]]):format(element.attr.name,element.attr.name))
			-- xml:add_child(reaction)
			-- reaction = nxml.parse(([[<Reaction probability="100" input_cell1="%s" input_cell2="[sand_other]" output_cell1="[sand_other]" output_cell2="%s" direction="top"></Reaction>]]):format(element.attr.name,element.attr.name))
			-- xml:add_child(reaction)
			-- reaction = nxml.parse(([[<Reaction probability="100" input_cell1="%s" input_cell2="air" output_cell1="air" output_cell2="%s" direction="left"></Reaction>]]):format(element.attr.name, element.attr.name))
			-- xml:add_child(reaction)
			-- reaction = nxml.parse(([[<Reaction probability="100" input_cell1="%s" input_cell2="air" output_cell1="air" output_cell2="%s" direction="right"></Reaction>]]):format(element.attr.name, element.attr.name))
			-- xml:add_child(reaction)
			
			local color
			local graphics = element:first_of("Graphics")

			if graphics then
				color = graphics.attr.color or element.attr.wang_color
			else
				graphics = nxml.parse(([[<Graphics color="%s"></Graphics>]]):format(element.attr.wang_color))
				element.add_child(graphics)
				color = graphics.attr.color
			end

			if #color == 8 then
				local alpha_string = color:sub(1, 2)
				local alpha = tonumber(alpha_string, 16)
				alpha = math.min(255, alpha + 255)
				alpha_string = string.format("%.2X", alpha)
				color = alpha_string .. color:sub(3, -1)
			end
			graphics.attr.color = color
			
			excluded_materials[element.attr.name .. "_inverse_gravity_gas_form"] = true
			element.attr.density = 700 - (element.attr.density or 1)
		elseif ( get_setting("inverse_gravity.gas_gravity") or get_setting("inverse_gravity.liquid_gravity") or get_setting("inverse_gravity.sand_gravity")) then
			element.attr.density = 700 + (element.attr.density or 1)
		end
		excluded_materials[element.attr.name] = true
	end
	
	if get_setting("inverse_gravity.liquid_gravity") then
		--make potions suck up gases
		replace_value("data/entities/items/pickup/potion.xml", 
			[[%<MaterialSuckerComponent]], 
			[[<MaterialSuckerComponent
			material_type="2"
			suck_tag="[liquid_converted_gravity]"]]
		)
		
		replace_value("data/entities/items/pickup/potion.xml", 
			[[%<PotionComponent]], 
			[[<PotionComponent
			dont_spray_just_leak_gas_materials="0"]]
		)
		
		if (element.attr.name == "blood_fungi") then
			element.attr.lifetime = "120"
		elseif (element.attr.name == "blood_cold") then
			element.attr.lifetime = "240"
		end
	end
	function has_liquid_static_parent(element)
	  while element do
		  if element.attr.liquid_static == "1" then
			  return true
		  end
		  element = lookup_by_name_orig[element.attr._parent]
	  end
	  return false
	end
	
	function has_liquid_cell_parent(element)
	  while element do
		  if element.attr.cell_type == "liquid" then
			  return true
		  end
		  element = lookup_by_name_orig[element.attr._parent]
	  end
	  return false
	end
	
	function has_liquid_sand_parent(element)
	  while element do
		  if element.attr.liquid_sand == "1" then
			  return true
		  end
		  element = lookup_by_name_orig[element.attr._parent]
	  end
	  return false
	end
	
	function check_liquid_chain(element)
		while element do
			if ( has_liquid_cell_parent(element) == true ) and 
				( has_liquid_static_parent(element) == false ) and
				( has_liquid_sand_parent(element) == false) then
				return true
			end
			element = lookup_by_name_orig[element.attr._parent]
		end
	end

	if ( check_liquid_chain(element) and not excluded_materials[element.attr.name] ) or ( element.attr.name == "peat" )then
		if get_setting("inverse_gravity.liquid_gravity") then
			element.attr.cell_type = "gas"
			element.attr.liquid_flow_speed = 1
			element.attr.liquid_stains = 3
			
			--PLEASE CHANGE THIS WHEN BETA GETS MERGED!!!
			-- if is_in_beta == true then
				element.attr.gas_speed = 250
			-- end

			-- element.attr.gas_speed = 50
			-- element.attr.gas_upwards_speed = 100
			-- element.attr.gas_downwards_speed = 90
			-- element.attr.gas_horizontal_speed = 100
			
			if element.attr.tags then
			  element.attr.tags = element.attr.tags .. ",[liquid_converted_gravity]"
			else
			  element.attr.tags = "[liquid_converted_gravity]"
			end

			if element.attr.name ~= "oil" then
				element.attr.electrical_conductivity = 1
			end
			
			local color
			local graphics = element:first_of("Graphics")

			if graphics then
				color = graphics.attr.color or element.attr.wang_color
			else
				graphics = nxml.parse(([[<Graphics color="%s"></Graphics>]]):format(element.attr.wang_color))
				element.add_child(graphics)
				color = graphics.attr.color
			end

			if #color == 8 then
				local alpha_string = color:sub(1, 2)
				local alpha = tonumber(alpha_string, 16)
				alpha = math.min(255, alpha + 55)
				alpha_string = string.format("%.2X", alpha)
				color = alpha_string .. color:sub(3, -1)
			end
			graphics.attr.color = color
			
			local status_effects = element:first_of("StatusEffects")

			if status_effects then
				local ingestion = status_effects:first_of("Ingestion")
				if not ingestion then
					ingest_status = nxml.parse([[<Ingestion><StatusEffect type="NONE" amount="0" /></Ingestion>]])
					status_effects:add_child(ingest_status)
				end
			else
				ingest = nxml.parse([[<StatusEffects><Ingestion><StatusEffect type="NONE" amount="0" /></Ingestion></StatusEffects>]])
				element:add_child(ingest)
			end
			
			element.attr.density = 800 - (element.attr.density or 1)
		elseif ( get_setting("inverse_gravity.gas_gravity") or get_setting("inverse_gravity.liquid_gravity") or get_setting("inverse_gravity.sand_gravity")) then
			element.attr.density = 800 + (element.attr.density or 1)
		end
		excluded_materials[element.attr.name] = true
	end

	function check_gas_chain(element)
		while element do
			if ( element.attr.name ~= "cloud" ) and 
				( element.attr.name ~= "cloud_lighter" ) and 
				( element.attr.cell_type == "gas" ) and 
				( element.attr.liquid_sand == "0") then
				return true
			end
			element = lookup_by_name_orig[element.attr._parent]
		end
	end

	if check_gas_chain(element) and not excluded_materials[element.attr.name] then
		if get_setting("inverse_gravity.gas_gravity") then
			element.attr.cell_type = "liquid"
			element.attr.liquid_gravity = 0
			element.attr.liquid_flow_speed = 0.1
			element.attr.electrical_conductivity = 0
			
			local color
			local graphics = element:first_of("Graphics")

			if graphics then
				color = graphics.attr.color or element.attr.wang_color
			else
				graphics = nxml.parse(([[<Graphics color="%s"></Graphics>]]):format(element.attr.wang_color))
				element.add_child(graphics)
				color = graphics.attr.color
			end

			if #color == 8 then
				local alpha_string = color:sub(1, 2)
				local alpha = tonumber(alpha_string, 16)
				alpha = math.min(255, alpha - 55)
				alpha_string = string.format("%.2X", alpha)
				color = alpha_string .. color:sub(3, -1)
			end
			graphics.attr.color = color
			element.attr.density = 600 - (element.attr.density or 1)
		elseif ( get_setting("inverse_gravity.gas_gravity") or get_setting("inverse_gravity.liquid_gravity") or get_setting("inverse_gravity.sand_gravity")) then
			element.attr.density = 600 + (element.attr.density or 1)
		end
		
		excluded_materials[element.attr.name] = true
	end
	if not excluded_materials[element.attr.name] and ( get_setting("inverse_gravity.gas_gravity") or get_setting("inverse_gravity.liquid_gravity") or get_setting("inverse_gravity.sand_gravity")) then
		element.attr.density = (element.attr.density or 1) + 900
	end

end

if get_setting("inverse_gravity.sand_gravity") then
	reaction = nxml.parse([[<Reaction 
		probability="100" 
		input_cell1="[material_identifier]" input_cell2="air" 
		output_cell1="air" output_cell2="[material_identifier]_inverse_gravity_gas_form" 
		direction="top">
	</Reaction>]])
	xml:add_child(reaction)
end

local original_maximum = MagicNumbersGetValue( "CELLFACTORY_CELLDATA_MAX_COUNT" )

replace_value("data/magic_numbers.xml", [[%<MagicNumbers]], ([[<MagicNumbers 
  CELLFACTORY_CELLDATA_MAX_COUNT="%s"]]):format(original_maximum + sand_count), true)
-- Save the modified XML content back to the file
ModTextFileSetContent("data/materials.xml", tostring(xml))
-- print(tostring(xml))

local function entity_is_wand(entity_id)
	local ability_component = EntityGetFirstComponentIncludingDisabled(entity_id, "AbilityComponent")
	return ComponentGetValue2(ability_component, "use_gun_script") == true
end

if get_setting("inverse_gravity.liquid_gravity") then
	replace_value("data/scripts/buildings/teleport_liquid_check.lua", 
	[[false]], 
	[[true]]
	)
end

if mode == "inverted" then
		replace_value("data/scripts/perks/attract_items.lua", 
	[[PhysicsApplyForce%( item_id%, vel_x%, vel_y %)]], 
	[[PhysicsApplyForce( item_id, vel_x * 70, vel_y * 70 )]]
	)
end
if get_setting("inverse_gravity.flip_screen") then
	replace_value("data/shaders/post_final.vert", 
	[[tex_coord_ %= gl_TexCoord%[0%]%.xy%;]], 
	[[tex_coord_ = gl_TexCoord[0].xy;
	tex_coord_.y = 1.0 - tex_coord_.y; // add this]]
	)
	replace_value("data/shaders/post_final.vert", 
	[[tex_coord_y_inverted_ %= gl_TexCoord%[0%]%.zw %+ vec2%(camera_subpixel_offset%.x%, camera_subpixel_offset%.y%)%;]], 
	[[tex_coord_y_inverted_ = gl_TexCoord[0].zw + vec2(camera_subpixel_offset.x, camera_subpixel_offset.y);
	tex_coord_y_inverted_.y = 1.0 - tex_coord_y_inverted_.y; // add this]]
	)
	replace_value("data/shaders/post_final.vert", 
	[[tex_coord_glow_ %= gl_TexCoord%[1%]%.xy%;]], 
	[[tex_coord_glow_ = gl_TexCoord[1].xy;
	tex_coord_glow_.y = 1.0 - tex_coord_glow_.y; // add this]]
	)
end

if get_setting("inverse_gravity.player") then
	local player_content = ModTextFileGetContent("mods/inverse_gravity/player_base.xml")
	ModTextFileSetContent("data/entities/player_base.xml", player_content)
	
	local bubble_content = ModTextFileGetContent("mods/inverse_gravity/gas_bubble.xml")
	ModTextFileSetContent("data/entities/particles/gas_bubble.xml", bubble_content)
	
	replace_value("data/entities/misc/perks/shield.xml", 
	[[position%.y%=%"%-4%"]], 
	[[position.y="4"]]
	)
	replace_value("data/entities/misc/perks/contact_damage.xml", 
	[[position%.y%=%"%-4%"]], 
	[[position.y="4"]]
	)
	replace_value("data/entities/misc/perks/projectile_eater_sector.xml", 
	[[position%.y%=%"%-8%"]], 
	[[position.y="8"]]
	)
	replace_value("data/entities/misc/perks/projectile_eater_sector_noparticles.xml", 
	[[position%.y%=%"%-8%"]], 
	[[position.y="8"]]
	)
	replace_value("data/entities/misc/perks/projectile_repulsion_field.xml", 
	[[position%.y%=%"%-8%"]], 
	[[position.y="8"]]
	)
	replace_value("data/entities/misc/perks/projectile_repulsion_sector.xml", 
	[[position%.y%=%"%-8%"]], 
	[[position.y="8"]]
	)
	replace_value("data/entities/misc/perks/projectile_repulsion_sector_noparticles.xml", 
	[[position%.y%=%"%-8%"]], 
	[[position.y="8"]]
	)
	replace_value("data/entities/misc/perks/projectile_slow_field.xml", 
	[[position%.y%=%"%-4%"]], 
	[[position.y="4"]]
	)
end



if get_setting("inverse_gravity.item_gravity") then
	replace_value("data/scripts/items/generate_shop_item.lua", 
	[[image_file%=%"data%/fonts%/font_pixel_white%.xml%"%,]], 
	[[image_file= "data/fonts/font_pixel_white.xml",
	has_special_scale="1",]]
	)

	replace_value("data/scripts/items/generate_shop_item.lua", 
	[[image_file%=%"data%/fonts%/font_pixel_white%.xml%"%,]], 
	[[image_file= "data/fonts/font_pixel_white.xml",
	has_special_scale="1",]]
	)
end

if get_setting("inverse_gravity.item_gravity") then
	replace_value("data/scripts/items/heart.lua", 
	[[%-8]], 
	[[+8]]
	)

	replace_value("data/scripts/items/heart.lua", 
	[[%-12]], 
	[[+12]]
	)
	
	replace_value("data/scripts/items/heart_better.lua", 
	[[%-8]], 
	[[+8]]
	)

	replace_value("data/scripts/items/heart_better.lua", 
	[[%-12]], 
	[[+12]]
	)
	
	replace_value("data/scripts/items/heart_evil.lua", 
	[[%-12]], 
	[[+12]]
	)
	
	replace_value("data/scripts/items/heart_fullhp.lua", 
	[[%-8]], 
	[[+8]]
	)

	replace_value("data/scripts/items/heart_fullhp.lua", 
	[[%-12]], 
	[[+12]]
	)
	
	replace_value("data/scripts/items/heart_fullhp_temple.lua", 
	[[%-8]], 
	[[+8]]
	)

	replace_value("data/scripts/items/heart_fullhp_temple.lua", 
	[[%-12]], 
	[[+12]]
	)
	
	replace_value("data/scripts/items/spell_refresh.lua", 
	[[%-12]], 
	[[+12]]
	)
end

function OnWorldPostUpdate()
	-- GamePrint(MagicNumbersGetValue( "CELLFACTORY_CELLDATA_MAX_COUNT" ))
	-- GamePrint(sand_count)
	local cx, cy = GameGetCameraPos()
	
	local all_entities = EntityGetInRadius( cx, cy, 9999999999999999 )

	if ( #all_entities > 0 ) then
	
		for i, entity in ipairs(all_entities) do
		
			local ex, ey = EntityGetTransform( entity )

			if get_setting("inverse_gravity.flip_screen") then
				if (ex < cx + 256) or
					(ex > cx - 256) or
					(ey > cy - 150) or
					(ey < cy + 150) then
			
					local sprite_comps = EntityGetComponent(entity, "SpriteComponent") or {}
					
					for i, comp in ipairs(sprite_comps) do

						if ComponentGetValue2(comp, "emissive") == true then
						
							ComponentSetValue2(comp, "emissive", false)
							ComponentSetValue2(comp, "z_index", -50)
							EntityRefreshSprite(entity, comp)
							
						end
					end
					
					local sprite_particle_comps = EntityGetComponent(entity, "SpriteParticleEmitterComponent") or {}
					
					for i, comp in ipairs(sprite_particle_comps) do

						if ComponentGetValue2(comp, "emissive") == true then
						
							ComponentSetValue2(comp, "emissive", false)
							ComponentSetValue2(comp, "z_index", -50)
						end

					end
					
					function remove_emissive_from_explosion(entity_value, component_name)
						local val_comps = EntityGetComponent(entity_value, component_name) or {}
						
						for i, comp in ipairs(val_comps) do
								
							if ComponentObjectGetValue2( comp, "config_explosion", "explosion_sprite_emissive" ) == true then
									
								ComponentObjectSetValue2( comp, "config_explosion", "explosion_sprite_emissive", false )
							end
						end
					end
					
					remove_emissive_from_explosion(entity, "ProjectileComponent")
					remove_emissive_from_explosion(entity, "LightningComponent")
					-- remove_emissive_from_explosion(entity, "AnimalAIComponent")
					remove_emissive_from_explosion(entity, "ExplosionComponent")
					remove_emissive_from_explosion(entity, "ExplodeOnDamageComponent")
				end
			end
			
			if not EntityHasTag(entity, "inverted_gravity") then
			
				local vel_comp = EntityGetFirstComponentIncludingDisabled( entity, "VelocityComponent")
				local proj_comp = EntityGetFirstComponentIncludingDisabled( entity, "ProjectileComponent")
				local item_comp = EntityGetFirstComponentIncludingDisabled( entity, "ItemComponent")
				local phys_comp = EntityGetFirstComponentIncludingDisabled( entity, "PhysicsBodyComponent")
				local damage_comp = EntityGetFirstComponentIncludingDisabled( entity, "DamageModelComponent")
			
				if ( vel_comp ~= nil ) and ( vel_comp > 0 ) and ( proj_comp ~= nil ) and ( proj_comp > 0 ) and get_setting("inverse_gravity.projectile_gravity") then
					ComponentSetValue2(vel_comp, "gravity_y", ComponentGetValue2(vel_comp, "gravity_y") * -1)
					if get_setting("inverse_gravity.liquid_gravity") and get_setting("inverse_gravity.gas_gravity") then
						ComponentSetValue2(vel_comp, "displace_liquid", false)
						ComponentSetValue2(vel_comp, "liquid_drag", 0)
					end
					-- local x, y, rot, scale_x, scale_y  = EntityGetTransform(entity)
					-- EntitySetTransform(entity, x, y, rot, scale_x * -1, scale_x * -1)
				end
				if ( vel_comp ~= nil ) and ( vel_comp > 0 ) and ( item_comp ~= nil ) and ( item_comp > 0 ) and ( ( phys_comp == nil ) or ( phys_comp == 0 ) ) and get_setting("inverse_gravity.item_gravity") then
					ComponentSetValue2(vel_comp, "gravity_y", ComponentGetValue2(vel_comp, "gravity_y") * -1)
					local x, y, rot, scale_x, scale_y  = EntityGetTransform(entity)
					EntitySetTransform(entity, x, y, rot, scale_x * -1, scale_x * -1)
					EntityAddComponent2(entity, "LuaComponent", {
						script_throw_item="mods/inverse_gravity/physics_throw_fix1.lua",
						execute_every_n_frame=-1
					})
				end
				-- if ( phys_comp ~= nil ) and ( phys_comp > 0 ) and get_setting("inverse_gravity.player") then
					-- -- local throw_coeff = ComponentGetValue2(physics_throw_comp, "throw_force_coeff")
					-- -- ComponentSetValue2(physics_throw_comp, "throw_force_coeff", throw_coeff * -1)
					-- EntityAddComponent2(entity, "LuaComponent", {
						-- script_throw_item="mods/inverse_gravity/physics_throw_fix1.lua",
						-- execute_every_n_frame=-1
					-- })
				-- end
				if ( phys_comp ~= nil ) and ( phys_comp > 0 ) and mode == "inverted" then
					ComponentSetValue2(phys_comp, "gravity_scale_if_has_no_image_shapes", ComponentGetValue2(phys_comp, "gravity_scale_if_has_no_image_shapes") * -1)
					ComponentSetValue2(phys_comp, "hax_fix_going_through_ground", false)
				end
				if ( damage_comp ~= nil ) and ( damage_comp > 0 ) and get_setting("inverse_gravity.liquid_gravity") then
					ComponentSetValue2(damage_comp, "ragdoll_blood_amount_absolute", ComponentGetValue2(damage_comp, "ragdoll_blood_amount_absolute") * 0.2)
				end
				EntityAddTag( entity, "inverted_gravity" )
				
			end
		end
	end
end
-- function OnWorldPreUpdate()
	-- local cx, cy = GameGetCameraPos()
	
	-- local all_entities = EntityGetInRadius( cx, cy, 500 )

	-- if ( #all_entities > 0 ) then
	
		-- for i, entity in ipairs(all_entities) do
		
			-- -- local ex, ey = EntityGetTransform( entity )

			-- -- if get_setting("inverse_gravity.flip_screen") then
				-- -- if (ex < cx + 256) or
					-- -- (ex > cx - 256) or
					-- -- (ey > cy - 150) or
					-- -- (ey < cy + 150) then
			
					-- -- local sprite_comps = EntityGetComponent(entity, "SpriteComponent") or {}
					
					-- -- for i, comp in ipairs(sprite_comps) do

						-- -- if ComponentGetValue2(comp, "emissive") == true then
						
							-- -- ComponentSetValue2(comp, "emissive", false)
							-- -- ComponentSetValue2(comp, "z_index", -50)
							-- -- EntityRefreshSprite(entity, comp)
							
						-- -- end
					-- -- end
					
					-- -- local sprite_particle_comps = EntityGetComponent(entity, "SpriteParticleEmitterComponent") or {}
					
					-- -- for i, comp in ipairs(sprite_particle_comps) do

						-- -- if ComponentGetValue2(comp, "emissive") == true then
						
							-- -- ComponentSetValue2(comp, "emissive", false)
							-- -- ComponentSetValue2(comp, "z_index", -50)
						-- -- end

					-- -- end
					
					-- -- function remove_emissive_from_explosion(entity_value, component_name)
						-- -- local val_comps = EntityGetComponent(entity_value, component_name) or {}
						
						-- -- for i, comp in ipairs(val_comps) do
								
							-- -- if ComponentObjectGetValue2( comp, "config_explosion", "explosion_sprite_emissive" ) == true then
									
								-- -- ComponentObjectSetValue2( comp, "config_explosion", "explosion_sprite_emissive", false )
							-- -- end
						-- -- end
					-- -- end
					
					-- -- remove_emissive_from_explosion(entity, "ProjectileComponent")
					-- -- remove_emissive_from_explosion(entity, "LightningComponent")
					-- -- -- remove_emissive_from_explosion(entity, "AnimalAIComponent")
					-- -- remove_emissive_from_explosion(entity, "ExplosionComponent")
					-- -- remove_emissive_from_explosion(entity, "ExplodeOnDamageComponent")
				-- -- end
			-- -- end

			-- if not EntityHasTag(entity, "inverted_gravity") then
			
				-- local vel_comp = EntityGetFirstComponentIncludingDisabled( entity, "VelocityComponent")
				-- local proj_comp = EntityGetFirstComponentIncludingDisabled( entity, "ProjectileComponent")
				-- local item_comp = EntityGetFirstComponentIncludingDisabled( entity, "ItemComponent")
				-- local phys_comp = EntityGetFirstComponentIncludingDisabled( entity, "PhysicsBodyComponent")
				-- local damage_comp = EntityGetFirstComponentIncludingDisabled( entity, "DamageModelComponent")
			
				-- if ( vel_comp ~= nil ) and ( vel_comp > 0 ) and ( proj_comp ~= nil ) and ( proj_comp > 0 ) and get_setting("inverse_gravity.projectile_gravity") then
					-- ComponentSetValue2(vel_comp, "gravity_y", ComponentGetValue2(vel_comp, "gravity_y") * -1)
					-- -- local x, y, rot, scale_x, scale_y  = EntityGetTransform(entity)
					-- -- EntitySetTransform(entity, x, y, rot, scale_x * -1, scale_x * -1)
				-- end
				-- if ( vel_comp ~= nil ) and ( vel_comp > 0 ) and ( item_comp ~= nil ) and ( item_comp > 0 ) and ( ( phys_comp == nil ) or ( phys_comp == 0 ) ) and get_setting("inverse_gravity.item_gravity") then
					-- ComponentSetValue2(vel_comp, "gravity_y", ComponentGetValue2(vel_comp, "gravity_y") * -1)
					-- local x, y, rot, scale_x, scale_y  = EntityGetTransform(entity)
					-- EntitySetTransform(entity, x, y, rot, scale_x * -1, scale_x * -1)
					-- EntityAddComponent2(entity, "LuaComponent", {
						-- script_throw_item="mods/inverse_gravity/physics_throw_fix1.lua",
						-- execute_every_n_frame=-1
					-- })
				-- end
				-- -- if ( phys_comp ~= nil ) and ( phys_comp > 0 ) and get_setting("inverse_gravity.player") then
					-- -- -- local throw_coeff = ComponentGetValue2(physics_throw_comp, "throw_force_coeff")
					-- -- -- ComponentSetValue2(physics_throw_comp, "throw_force_coeff", throw_coeff * -1)
					-- -- EntityAddComponent2(entity, "LuaComponent", {
						-- -- script_throw_item="mods/inverse_gravity/physics_throw_fix1.lua",
						-- -- execute_every_n_frame=-1
					-- -- })
				-- -- end
				-- if ( phys_comp ~= nil ) and ( phys_comp > 0 ) and get_setting("inverse_gravity.physics_gravity") then
					-- ComponentSetValue2(phys_comp, "hax_fix_going_through_ground", false)
				-- end
				-- if ( damage_comp ~= nil ) and ( damage_comp > 0 ) and get_setting("inverse_gravity.liquid_gravity") then
					-- ComponentSetValue2(damage_comp, "ragdoll_blood_amount_absolute", ComponentGetValue2(damage_comp, "ragdoll_blood_amount_absolute") * 0.2)
				-- end
				-- EntityAddTag( entity, "inverted_gravity" )
				
			-- end
		-- end

	-- end
-- end