dofile_once( "data/scripts/lib/utilities.lua" )

function get_distance( x1, y1, x2, y2 )
	local result = math.sqrt( ( x2 - x1 ) ^ 2 + ( y2 - y1 ) ^ 2 )
	return result
end

function approx_rolling_average(avg, target, n)
  avg = avg - avg / n;
  avg = avg + target / n;
  return avg
end

setting_cache = setting_cache or {}
local function get_setting(name)
    setting_cache[name] = setting_cache[name] or ModSettingGet(name)
    return setting_cache[name]
end

local entity_id = GetUpdatedEntityID()
local ex, ey, erot, escale_x  = EntityGetTransform(entity_id)
local parent_id = EntityGetParent( entity_id )
local x, y, rot, scale_x  = EntityGetTransform(parent_id)

local total_accumulated_gold = 0

if (total_accumulated_gold > 0) then
	local wallet = EntityGetFirstComponent(entity_id, "WalletComponent")
	ComponentSetValue2(wallet, "money", ComponentGetValue2(wallet, "money") + total_accumulated_gold)
end

local character_component = EntityGetFirstComponentIncludingDisabled(entity_id, "CharacterDataComponent")
local character_component_parent = EntityGetFirstComponentIncludingDisabled(parent_id, "CharacterDataComponent")

local flight_val = ComponentGetValue2(character_component_parent, "mFlyingTimeLeft")
ComponentSetValue2( character_component, "mFlyingTimeLeft", flight_val )

local did_hit, hit_x, hit_y = RaytracePlatforms(x - 2, y - 4 , x + 2, y - 4)

local did_hit2, hit_x2, hit_y2 = RaytracePlatforms(x - 2, y + 9 , x + 2, y + 9)

if ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted") < 150 then
	EntitySetTransform(entity_id, x, y)
else
	if get_distance(ex, ey, x, y) < 20 then
		EntitySetTransform(parent_id, ex, ey)
	else
		EntitySetTransform(entity_id, x, y)
	end
end

if did_hit2 and ( not did_hit ) then
	EntitySetTransform(parent_id, ex, ey - 1)
	EntitySetTransform(entity_id, ex, ey - 1)
end

local old_value_sprite = GetValueBool("value_sprite", 0)

local on_land = false
if did_hit then
	ComponentSetValue2( character_component, "is_on_ground", true )
	ComponentSetValue2( character_component_parent, "is_on_ground", true )
	on_land = true
else
	ComponentSetValue2( character_component, "is_on_ground", false )
	ComponentSetValue2( character_component_parent, "is_on_ground", false )
end

local disable_movement = false

local cooldown_value = GetValueNumber("cooldown", 0)
SetValueNumber("cooldown", cooldown_value + 1)

local telekinesis_component = EntityGetFirstComponent(parent_id, "TelekinesisComponent")

for i, child in ipairs(EntityGetAllChildren(parent_id) or {}) do
	local game_effect_component = EntityGetFirstComponent(child, "GameEffectComponent")
	if game_effect_component then
		if ( ComponentGetValue2(game_effect_component, "disable_movement") == true ) then
			disable_movement = true
		elseif ( ComponentGetValue2(game_effect_component, "effect") == "KNOCKBACK" ) and ( cooldown_value < 60) then

			if on_land then
				SetValueNumber("cooldown", 60 )
			end
			disable_movement = true
		elseif cooldown_value > 240 then
			SetValueNumber("cooldown", 0 )
		end
	end
	if EntityGetName(child) == "gold_sucker" then

		local component_sucker = EntityGetFirstComponent(child, "MaterialSuckerComponent")
		local compovar_accumulated = EntityGetFirstComponent(child, "VariableStorageComponent")

		if (component_sucker ~= nil and compovar_accumulated ~= nil) then
			local accumulated_currently = ComponentGetValue2(component_sucker, "mGoldAccumulator")
			local accumulated_last_frame = ComponentGetValue2(compovar_accumulated, "value_int")

			if (accumulated_currently ~= accumulated_last_frame) then
				if (accumulated_currently > accumulated_last_frame) then
					total_accumulated_gold = total_accumulated_gold + (accumulated_currently - accumulated_last_frame)
				end
		
				ComponentSetValue(compovar_accumulated, "value_int", accumulated_currently)
			end
		end
	end
end

-- local invgui_component = EntityGetFirstComponentIncludingDisabled(entity_id, "InventoryGuiComponent")
local controls_component = EntityGetFirstComponentIncludingDisabled(entity_id, "ControlsComponent")
local controls_component_parent = EntityGetFirstComponentIncludingDisabled(parent_id, "ControlsComponent")

-- if InputIsMouseButtonDown() then
	-- ComponentSetValue2(player_component_parent, "mForceFireOnNextUpdate", true)
-- end

local character_platforming_component = EntityGetFirstComponentIncludingDisabled(entity_id, "CharacterPlatformingComponent")
local character_platforming_component_parent = EntityGetFirstComponentIncludingDisabled(parent_id, "CharacterPlatformingComponent")

local did_hit3, hit_x3, hit_y3 = RaytraceSurfacesAndLiquiform(x - 1, y + 3, x + 1, y + 3) --does not return gas and fire
local did_hit4, hit_x4, hit_y4 = Raytrace(x - 1, y + 3, x + 1, y + 3) -- returns any surface
local did_hit5, hit_x5, hit_y5 = RaytracePlatforms(x - 1, y + 3, x + 1, y + 3) -- returns any platformable surface

local did_hit6, hit_x6, hit_y6 = RaytraceSurfacesAndLiquiform(x, y + 6, x, y + 6) --does not return gas and fire
local did_hit7, hit_x7, hit_y7 = Raytrace(x-2, y + 6, x + 2, y + 6) -- returns any surface
local submerged = false

local damage_model_component = EntityGetFirstComponentIncludingDisabled(parent_id, "DamageModelComponent")

local is_on_fire = ComponentGetValue2(damage_model_component, "is_on_fire")
if (not did_hit6) and did_hit7 and get_setting("inverse_gravity.liquid_gravity") then
	submerged = true
end
if (not did_hit3) and did_hit4 and get_setting("inverse_gravity.liquid_gravity") then
	
	if ( ComponentGetValue2(controls_component, "mButtonDownDown") == false ) then
		local damage_component = EntityGetFirstComponentIncludingDisabled(entity_id, "DamageModelComponent")
		local damage_component_parent = EntityGetFirstComponentIncludingDisabled(parent_id, "DamageModelComponent")
		-- local tags_string = CellFactory_GetTags( ComponentGetValue2(damage_component, "mLiquidMaterialWeAreIn") )
		-- local found = (tags_string):find("%[liquid_converted_gravity%]") ~= nil
		-- if found then
		local velx, vely = ComponentGetValue2(character_component, "mVelocity")
		ComponentSetValue2(character_component, "mVelocity", velx * 0.9, vely + 7.2)
		ComponentSetValue2( character_platforming_component, "mFramesInAirCounter", 0 )
		ComponentSetValue2( character_platforming_component, "mFramesNotSwimming", 0 )
		ComponentSetValue2( character_platforming_component, "mFramesSwimming", ComponentGetValue2(character_platforming_component, "mFramesSwimming") + 1 )



		ComponentSetValue2(damage_component_parent, "mTotalCells", 50)
		ComponentSetValue2(damage_component_parent, "mLiquidCount", 50)
		-- end
	else
		local damage_component = EntityGetFirstComponentIncludingDisabled(entity_id, "DamageModelComponent")
		local damage_component_parent = EntityGetFirstComponentIncludingDisabled(parent_id, "DamageModelComponent")
		-- local tags_string = CellFactory_GetTags( ComponentGetValue2(damage_component, "mLiquidMaterialWeAreIn") )
		-- local found = (tags_string):find("%[liquid_converted_gravity%]") ~= nil
		-- if found then
		local velx, vely = ComponentGetValue2(character_component, "mVelocity")
		ComponentSetValue2(character_component, "mVelocity", velx * 0.9, vely + 5)

		ComponentSetValue2( character_platforming_component, "mFramesInAirCounter", 0 )
		ComponentSetValue2( character_platforming_component, "mFramesNotSwimming", 0 )
		ComponentSetValue2( character_platforming_component, "mFramesSwimming", ComponentGetValue2(character_platforming_component, "mFramesSwimming") + 1 )



		ComponentSetValue2(damage_component_parent, "mTotalCells", 50)
		ComponentSetValue2(damage_component_parent, "mLiquidCount", 50)
	end
elseif ( (not did_hit5) and did_hit4 and get_setting("inverse_gravity.gas_gravity") ) or (is_on_fire) then
	local velx, vely = ComponentGetValue2(character_component, "mVelocity")
	ComponentSetValue2(character_component, "mVelocity", velx, vely - 7.2)
end

local climb_over_y_left = RaytracePlatforms(ex - 3.5, ey, ex - 3.5, ey - 2)
local climb_over_y_right = RaytracePlatforms(ex + 3.5, ey, ex + 3.5, ey - 2)
local collision_y_left = RaytracePlatforms(ex - 3.5, ey, ex - 3.5, ey + 4.5)
local collision_y_right = RaytracePlatforms(ex + 3.5, ey, ex + 3.5, ey + 4.5)
if ( (climb_over_y_left and not collision_y_left ) or (climb_over_y_right and not collision_y_right ) ) and ( ( ComponentGetValue2(controls_component, "mButtonDownLeft") == true ) or ( ComponentGetValue2(controls_component, "mButtonDownRight") == true ) ) then
	local ex, ey, erot, escale_x  = EntityGetTransform(entity_id)
	EntitySetTransform(entity_id, ex, ey + 1)
	EntitySetTransform(parent_id, ex, ey + 1)
end
local sprite_component = EntityGetFirstComponentIncludingDisabled(entity_id, "SpriteComponent")
local sprite_component_parent = EntityGetFirstComponentIncludingDisabled(parent_id, "SpriteComponent")

local x1, y1 = GameGetCameraPos()
local mx, my = DEBUG_GetMouseWorld()
-- ComponentSetValue2( controls_component_parent, "mMousePosition", mx, y1 - my )
local dirx, diry = ComponentGetValue2(controls_component, "mAimingVector")


-- local diry_val_old = GetValueNumber("diry_val", 0)

local player_component = EntityGetFirstComponentIncludingDisabled(entity_id, "PlatformShooterPlayerComponent")
local player_component_parent = EntityGetFirstComponentIncludingDisabled(parent_id, "PlatformShooterPlayerComponent")

if get_setting("inverse_gravity.flip_screen") then
	local dist2 = get_distance(x1, y1, mx, my)
	local scaleFactor = 4
	local offset = 1 / dist2 -- Invert the offset calculation
	diry = (y1 - ey - diry) + (scaleFactor * offset) 
	-- + ( offset * 4 )
	-- SetValueNumber("diry_val", diry )
	-- dirx = x1 - ex + dirx
end
local dirnormx, dirnormy = vec_normalize(dirx, diry)

--THIS IS PlayerShooterFuckerComponent
ComponentSetValue2(player_component_parent, "mForceFireOnNextUpdate", ComponentGetValue2(player_component, "mForceFireOnNextUpdate"))
ComponentSetValue2(player_component_parent, "mRequireTriggerPull", ComponentGetValue2(player_component, "mRequireTriggerPull"))

local vecx, vecy = ComponentGetValue2(player_component_parent, "mSmoothedAimingVector")

local vec_diry = diry * 0.004
local vec_dir_avg = lerp(vecy, vec_diry, 0.97)
ComponentSetValue2( player_component_parent, "mSmoothedAimingVector", vecx , vec_dir_avg)

if ComponentGetValue2(controls_component, "mButtonDownChangeItemL") then
	ComponentSetValue2(controls_component_parent, "mButtonDownChangeItemL", true)
	ComponentSetValue2(controls_component_parent, "mButtonFrameChangeItemL", GameGetFrameNum() + 1)
	ComponentSetValue2(controls_component_parent, "mButtonCountChangeItemL", 1)
end

if ComponentGetValue2(controls_component, "mButtonDownChangeItemR") then
	ComponentSetValue2(controls_component_parent, "mButtonDownChangeItemR", true)
	ComponentSetValue2(controls_component_parent, "mButtonFrameChangeItemR", GameGetFrameNum() + 1)
	ComponentSetValue2(controls_component_parent, "mButtonCountChangeItemR", 1)
end


if ( GetValueBool("value_interact", 0) == false ) and ( ComponentGetValue2(controls_component, "mButtonDownInteract") == true ) then
	ComponentSetValue2(controls_component_parent, "mButtonDownInteract", true)
	ComponentSetValue2(controls_component_parent, "mButtonFrameInteract", GameGetFrameNum() + 1)
	SetValueBool("value_interact", true)
elseif ( ComponentGetValue2(controls_component, "mButtonDownInteract") == false ) then
	SetValueBool("value_interact", false)
end

if ( GetValueBool("value_inventory", 0) == false ) and ( ComponentGetValue2(controls_component, "mButtonDownInventory") == true ) then
	ComponentSetValue2(controls_component_parent, "mButtonDownInventory", true)
	ComponentSetValue2(controls_component_parent, "mButtonFrameInventory", GameGetFrameNum() + 1)
	SetValueBool("value_inventory", true)
elseif ( ComponentGetValue2(controls_component, "mButtonDownInventory") == false ) then
	SetValueBool("value_inventory", false)
end

local animation = ComponentGetValue2( sprite_component, "rect_animation")
local animation_parent = ComponentGetValue2( sprite_component_parent, "rect_animation")

if (animation == "throw") or (animation_parent == "throw") or (animation == "kick") or (animation == "eat") or (animation_parent == "kick") or (animation_parent == "eat") then
	for k, v in pairs(ComponentGetMembers( sprite_component ))do
		if (k == "alpha") then
			ComponentSetValue2( sprite_component_parent, "alpha", 1 )
		elseif (k ~= "mRenderList") and (k ~= "mSprite") and (k~="mRenderListHandle") and (k ~= "rect_animation") and (k ~= "next_rect_animation") then
			ComponentSetValue2(sprite_component_parent, k, ComponentGetValue2(sprite_component, k))
		end
	end
else
	for k, v in pairs(ComponentGetMembers( sprite_component ))do
		if (k == "alpha") then
			ComponentSetValue2( sprite_component_parent, "alpha", 1 )
		elseif (k ~= "mRenderList") and (k ~= "mSprite") and (k~="mRenderListHandle") then
			ComponentSetValue2(sprite_component_parent, k, ComponentGetValue2(sprite_component, k))
		end
	end
end

local animation = ComponentGetValue2( sprite_component, "rect_animation")
local animation_parent = ComponentGetValue2( sprite_component_parent, "rect_animation")

if ( telekinesis_component ~= nil ) and ( telekinesis_component > 0 ) then
	if ( old_value_sprite == false ) and ( ComponentGetValue2(controls_component, "mButtonDownKick") == true ) then
		ComponentSetValue2(controls_component_parent, "mButtonDownKick", true)
		ComponentSetValue2(telekinesis_component, "mInteract", true)
		SetValueBool("value_sprite", true)
	elseif ( ComponentGetValue2(controls_component, "mButtonDownKick") == false ) then
		SetValueBool("value_sprite", false)
	end
else
	if ( ComponentGetValue2(controls_component, "mButtonDownKick") == true ) and ( old_value_sprite == false ) and ( animation ~= "kick" ) then
		ComponentSetValue2(controls_component_parent, "mButtonDownKick", true)
		ComponentSetValue2(controls_component_parent, "mButtonFrameKick", GameGetFrameNum() + 1)
		-- if ComponentHasTag( sprite_component, "character" ) then
			-- ComponentRemoveTag( sprite_component, "character" )
		-- end
		-- if ComponentHasTag( sprite_component_parent, "character" ) then
			-- ComponentRemoveTag( sprite_component_parent, "character" )
		-- end
		-- ComponentSetValue2(sprite_component, "rect_animation", "kick")
		-- ComponentSetValue2(sprite_component, "next_rect_animation", "stand")
		-- ComponentSetValue2(sprite_component_parent, "rect_animation", "kick")
		-- ComponentSetValue2(sprite_component_parent, "next_rect_animation", "stand")
		
		SetValueBool("value_sprite", true)

	elseif ( animation ~= "kick" ) and ( old_value_sprite == true ) and ( ComponentGetValue2(controls_component, "mButtonDownKick") == false ) then

		-- if not ComponentHasTag( sprite_component, "character" ) then
			-- ComponentAddTag( sprite_component, "character" )
		-- end
		-- if not ComponentHasTag( sprite_component_parent, "character" ) then
			-- ComponentAddTag( sprite_component_parent, "character" )
		-- end
		
		SetValueBool("value_sprite", false)
	end
end

function get_active_item()
	local inv = EntityGetFirstComponentIncludingDisabled(parent_id, "Inventory2Component")
	return ComponentGetValue2(inv, "mActiveItem")   
end

local active_item = get_active_item()
if ( active_item > 0 )  then
	local potion_comp = EntityGetFirstComponentIncludingDisabled( active_item, "PotionComponent" )
	if ComponentGetValue2(controls_component, "mButtonDownRightClick") and ( potion_comp ~= nil ) and ( potion_comp > 0 )then
		-- local ability_comp = EntityGetFirstComponentIncludingDisabled( active_item, "AbilityComponent" )
		local phys_comp = EntityGetFirstComponentIncludingDisabled( active_item, "PhysicsBodyComponent")
		if ( phys_comp ~= nil ) and ( phys_comp > 0 ) and (animation ~= "kick") and (animation ~= "throw") and ( old_value_sprite == false ) then
			ComponentSetValue2( controls_component_parent, "mButtonDownThrow", true )
			ComponentSetValue2( controls_component_parent, "mButtonFrameThrow", GameGetFrameNum() + 1 )
			-- ComponentSetValue2( ability_comp, "simulate_throw_as_item", true )
			-- if ComponentHasTag( sprite_component, "character" ) then
				-- ComponentRemoveTag( sprite_component, "character" )
			-- end
			-- if ComponentHasTag( sprite_component_parent, "character" ) then
				-- ComponentRemoveTag( sprite_component_parent, "character" )
			-- end
			-- ComponentSetValue2(sprite_component, "rect_animation", "throw")
			-- ComponentSetValue2(sprite_component, "next_rect_animation", "stand")
			-- ComponentSetValue2(sprite_component_parent, "rect_animation", "throw")
			-- ComponentSetValue2(sprite_component_parent, "next_rect_animation", "stand")
			SetValueBool("value_sprite", true)
		elseif ( animation ~= "throw" ) and ( old_value_sprite == true ) and ( ComponentGetValue2(controls_component, "mButtonDownFire2") == false ) then
			-- if not ComponentHasTag( sprite_component, "character" ) then
				-- ComponentAddTag( sprite_component, "character" )
			-- end
			-- if not ComponentHasTag( sprite_component_parent, "character" ) then
				-- ComponentAddTag( sprite_component_parent, "character" )
			-- end
			SetValueBool("value_sprite", false)
		end
	elseif ( ComponentGetValue2(controls_component, "mButtonDownFire") == true ) and (( potion_comp == nil ) or ( potion_comp == 0 )) then
		local phys_comp = EntityGetFirstComponentIncludingDisabled( active_item, "PhysicsBodyComponent")
		if ( phys_comp ~= nil ) and ( phys_comp > 0 ) and animation ~= "kick" and animation ~= "throw" then
			ComponentSetValue2( controls_component_parent, "mButtonDownThrow", true )
			ComponentSetValue2( controls_component_parent, "mButtonFrameThrow", GameGetFrameNum() + 1 )
			-- if ComponentHasTag( sprite_component, "character" ) then
				-- ComponentRemoveTag( sprite_component, "character" )
			-- end
			-- if ComponentHasTag( sprite_component_parent, "character" ) then
				-- ComponentRemoveTag( sprite_component_parent, "character" )
			-- end
			-- ComponentSetValue2(sprite_component, "rect_animation", "throw")
			-- ComponentSetValue2(sprite_component, "next_rect_animation", "stand")
			-- ComponentSetValue2(sprite_component_parent, "rect_animation", "throw")
			-- ComponentSetValue2(sprite_component_parent, "next_rect_animation", "stand")
			SetValueBool("value_sprite", true)
		elseif ( animation ~= "throw" ) and ( old_value_sprite == true ) and ( ComponentGetValue2(controls_component, "mButtonDownFire") == false ) then
			-- if not ComponentHasTag( sprite_component, "character" ) then
				-- ComponentAddTag( sprite_component, "character" )
			-- end
			-- if not ComponentHasTag( sprite_component_parent, "character" ) then
				-- ComponentAddTag( sprite_component_parent, "character" )
			-- end
			SetValueBool("value_sprite", false)
		end
	end
end

-- if (ComponentGetValue2(controls_component, "mButtonDownInventory") == true ) or
	-- (ComponentGetValue2(controls_component, "mButtonDownInteract") == true ) then
	
	
	-- local old_value = GetValueNumber("value", 0)
	-- SetValueNumber("value", old_value + 1)
	
	-- -- if ComponentGetValue2(controls_component, "mButtonDownChangeItemL") then
		-- -- SetValueBool("change_item_l_pressed", true)
	-- -- end
	
	-- -- if ComponentGetValue2(controls_component, "mButtonDownChangeItemR") then
		-- -- SetValueBool("change_item_r_pressed", true)
	-- -- end
	
	-- for k, v in pairs(ComponentGetMembers( controls_component ))do

		-- if (k == "mAimingVector") then
			-- ComponentSetValue2( controls_component_parent, "mAimingVector", dirx, diry )
		-- elseif (k == "mAimingVectorNormalized") then
			-- ComponentSetValue2( controls_component_parent, "mAimingVectorNormalized", dirnormx, dirnormy )
		-- -- elseif (k == "mAimingVectorNonZeroLatest") then
			-- -- ComponentSetValue2( controls_component_parent, "mAimingVectorNonZeroLatest", dirx, diry )
		-- -- elseif (k == "mGamepadAimingVectorRaw") then
			-- -- ComponentSetValue2( controls_component_parent, "mGamepadAimingVectorRaw", dirx, diry )
		-- -- elseif (k == "mMousePosition") then
			-- -- ComponentSetValue2( controls_component_parent, "mMousePosition", mx, y1 - my )
			-- -- ComponentSetValue2( controls_component, "mMousePosition", mx, y1 - my )
		-- -- elseif (k == "mMousePositionRaw") then
			-- -- ComponentSetValue2( controls_component_parent, "mMousePositionRaw", mx, y1 - my )
			-- -- ComponentSetValue2( controls_component, "mMousePositionRaw", mx, y1 - my )
		-- -- elseif (k == "mMousePositionRawPrev") then
			-- -- ComponentSetValue2( controls_component_parent, "mMousePositionRawPrev", mx, y1 - my )
			-- -- ComponentSetValue2( controls_component, "mMousePositionRawPrev", mx, y1 - my )
		-- elseif (k == "enabled") then
			-- ComponentSetValue2( controls_component_parent, "enabled", true )
		-- elseif (k~="mButtonDownThrow") and (k~="mButtonDownInteract") and (k~="mButtonDownInventory") and (k~="mButtonDownChangeItemR") and (k~="mButtonDownChangeItemL") then
			-- ComponentSetValue2(controls_component_parent, k, ComponentGetValue2(controls_component, k))
		-- end
	-- end
-- else
	-- SetValueNumber("value", 0)
	for k, v in pairs(ComponentGetMembers( controls_component ))do
		if (k == "mAimingVector") then
			ComponentSetValue2( controls_component_parent, "mAimingVector", dirx, diry )
		elseif (k == "mAimingVectorNormalized") then
			ComponentSetValue2( controls_component_parent, "mAimingVectorNormalized", dirnormx, dirnormy )
		-- elseif (k == "mAimingVectorNonZeroLatest") then
			-- ComponentSetValue2( controls_component_parent, "mAimingVectorNonZeroLatest", dirx, diry )
			-- GamePrint(ComponentGetValue2(controls_component, k))
		-- elseif (k == "mGamepadAimingVectorRaw") then
			-- GamePrint(ComponentGetValue2(controls_component, k))
			-- ComponentSetValue2( controls_component_parent, "mGamepadAimingVectorRaw", dirx, diry )
		-- elseif (k == "mMousePosition") then
			-- ComponentSetValue2( controls_component_parent, "mMousePosition", mx, y1 - my )
			-- ComponentSetValue2( controls_component, "mMousePosition", mx, y1 - my )
		-- elseif (k == "mMousePositionRaw") then
			-- ComponentSetValue2( controls_component_parent, "mMousePositionRaw", mx, y1 - my )
			-- ComponentSetValue2( controls_component, "mMousePositionRaw", mx, y1 - my )
		-- elseif (k == "mMousePositionRawPrev") then
			-- ComponentSetValue2( controls_component_parent, "mMousePositionRawPrev", mx, y1 - my )
			-- ComponentSetValue2( controls_component, "mMousePositionRawPrev", mx, y1 - my )
		elseif (k == "enabled") then
			ComponentSetValue2( controls_component_parent, "enabled", false )
		elseif (k~="mButtonDownThrow") and
				(k~="mButtonFrameThrow") and
				(k~="mButtonDownKick") and	
				(k~="mButtonFrameKick") and
				(k~="mButtonDownInteract") and	
				(k~="mButtonFrameInteract") and
				(k~="mButtonDownInventory") and	
				(k~="mButtonFrameInventory") and
				(k~="mButtonDownChangeItemR") and	
				(k~="mButtonCountChangeItemR") and
				(k~="mButtonFrameChangeItemR") and	
				(k~="mButtonDownChangeItemL") and	
				(k~="mButtonCountChangeItemL") and
				(k~="mButtonFrameChangeItemL")
				then
			ComponentSetValue2(controls_component_parent, k, ComponentGetValue2(controls_component, k))
		end
	end
-- end

-- if GetValueNumber("value", 0) > 1 then

	-- if ComponentGetValue2(controls_component, "mButtonDownInteract") then
		-- ComponentSetValue2(controls_component_parent, "mButtonDownInteract", true)
	-- end

	-- if ComponentGetValue2(controls_component, "mButtonDownInventory") then
		-- ComponentSetValue2(controls_component_parent, "mButtonDownInventory", true)
	-- end
-- end

local cooldown_eating_value = GetValueNumber("cooldown_eating", 0)
if (ComponentGetValue2(controls_component, "mButtonDownDown") == true ) and (ComponentGetValue2(controls_component, "mButtonDownKick") == false ) and on_land and (submerged == false) then
	-- SetValueNumber("cooldown_eating", cooldown_eating_value + 1)
		-- EntitySetComponentsWithTagEnabled(parent_id, "enabled_in_world", true)
		-- if ComponentHasTag( sprite_component, "character" ) == false then
			-- ComponentAddTag( sprite_component, "character" )
		-- end
		-- if ComponentHasTag( sprite_component_parent, "character" ) == false then
			-- ComponentAddTag( sprite_component_parent, "character" )
		-- end
		-- ComponentSetValue2(sprite_component, "rect_animation", "eat")
		-- ComponentSetValue2(sprite_component, "next_rect_animation", "stand")
		-- ComponentSetValue2(sprite_component_parent, "rect_animation", "eat")
		-- ComponentSetValue2(sprite_component_parent, "next_rect_animation", "stand")
		
		ComponentSetValue2(controls_component_parent, "mButtonDownEat", true)
		ComponentSetValue2(controls_component, "mButtonDownEat", true)
		ComponentSetValue2(player_component_parent, "mCrouching", true)

		-- function doit(mat_name)
		  -- local child = EntityCreateNew(mat_name)
		  -- EntitySetTransform(child, x, y)
		  -- EntityAddChild(parent_id, child)
		  -- local comp = EntityAddComponent2(child, "MaterialAreaCheckerComponent", {
			-- look_for_failure=false,
			-- kill_after_message=false,
			-- material=CellFactory_GetType(mat_name),
			-- material2=CellFactory_GetType(mat_name),
		  -- })
		  -- EntityAddComponent2(child, "InheritTransformComponent", {})
		  -- EntityAddComponent2(child, "LifetimeComponent", {lifetime=2})
		  -- EntityAddComponent2(child, "LuaComponent", {
			-- script_material_area_checker_success="mods/inverse_gravity/player_land.lua",
			-- execute_every_n_frame=-1,
		  -- })
		  -- ComponentSetValue2(comp, "area_aabb", 2, 0, 3, 1)
		-- end

		 -- local mats = {
		  -- CellFactory_GetAllGases( true, true ),
		-- }
		-- for i, v in ipairs(mats) do
		  -- for i, v2 in ipairs(v) do
			-- doit(v2)
		  -- end
		-- end


	-- end
-- elseif ( animation ~= "eat" ) and ( animation_parent ~= "eat" ) and (ComponentGetValue2(controls_component, "mButtonDownKick") == false ) and (ComponentGetValue2(controls_component, "mButtonDownDown") == false ) then
	-- SetValueNumber("cooldown_eating", 0)
	-- if ComponentHasTag( sprite_component, "character" ) == false then
		-- ComponentAddTag( sprite_component, "character" )
	-- end
	-- if ComponentHasTag( sprite_component_parent, "character" ) == false then
		-- ComponentAddTag( sprite_component_parent, "character" )
	-- end
	
	-- EntitySetComponentsWithTagEnabled(parent_id, "enabled_in_world", false)

end

-- function material_area_checker_success( pos_x, pos_y)
  -- local entity_id = GetUpdatedEntityID() 
  -- local parent_id = EntityGetParent(entity_id) 
  -- local x, y = EntityGetTransform(parent_id)
  -- local mat = EntityGetName(entity_id)
  -- local mat_id = CellFactory_GetType( mat )
  -- local mat_tags = CellFactory_GetTags( mat_id )
  -- for i, tag in ipairs(mat_tags or {}) do
	  -- if tag == "[liquid_converted_gravity]" then
		-- EntityIngestMaterial( parent_id, mat_id, 5 )
		-- GamePrint(mat_id)
		-- GamePrint(tag)
		
		-- local child = EntityCreateNew()
		-- EntitySetTransform(child, x, y)
		-- EntityAddChild(parent_id, child)
		-- EntityAddComponent2(child, "InheritTransformComponent", {})
		-- EntityAddComponent2(child, "LifetimeComponent", {lifetime=10})
		-- EntityAddComponent2(child, "LuaComponent", {
			-- script_source_file="mods/inverse_gravity/gas_eat.lua",
			-- execute_every_n_frame=1
		-- })
	  -- end
  -- end
  
-- end

--------CUSTOM AIMING----------
-- local vecx, vecy = ComponentGetValue2(player_component_parent, "mDesiredCameraPos")

-- local aiming_camera_range_modifier = 0.825
-- local aiming_camera_smoothness_modifier = 0.85

-- local avgx, avgy = vec_lerp(vecx, vecy, x + mx - x1, y + (my - y1 ) * -1, aiming_camera_range_modifier)

-- local x1, y1 = vec_lerp(x1, y1, avgx, avgy, aiming_camera_smoothness_modifier)

-- ComponentSetValue2( player_component_parent, "mDesiredCameraPos", x1, y1)
------------------------------

for k, v in pairs(ComponentGetMembers( character_component ))do
	if (k ~= "is_on_ground") then
		ComponentSetValue2(character_component_parent, k, ComponentGetValue2(character_component, k))
	end
end

if disable_movement == false then
	for k, v in pairs(ComponentGetMembers( character_platforming_component ))do
		if(k == "run_velocity") then
			ComponentSetValue2( character_platforming_component, "run_velocity", 154)
		elseif(k == "fly_speed_max_up") then
			ComponentSetValue2( character_platforming_component, "fly_speed_max_up", -95)
		elseif(k == "jump_velocity_y") then
			ComponentSetValue2( character_platforming_component, "jump_velocity_y", -95)
		elseif(k == "jump_velocity_x") then
			ComponentSetValue2( character_platforming_component, "jump_velocity_x", 56)
		elseif(k == "fly_velocity_x") then
			ComponentSetValue2( character_platforming_component, "fly_velocity_x", 52)
		elseif(k == "fly_speed_mult") then
			ComponentSetValue2( character_platforming_component, "fly_speed_mult", 20)
		elseif(k == "fly_speed_change_spd") then
			ComponentSetValue2( character_platforming_component, "fly_speed_change_spd", 0.25)
		else
			ComponentSetValue2(character_platforming_component_parent, k, ComponentGetValue2(character_platforming_component, k))
		end
	end
else
	for k, v in pairs(ComponentGetMembers( character_platforming_component ))do
		if(k == "run_velocity") then
			ComponentSetValue2( character_platforming_component, "run_velocity", 0)
		elseif(k == "fly_speed_max_up") then
			ComponentSetValue2( character_platforming_component, "fly_speed_max_up", 0)
		elseif(k == "jump_velocity_y") then
			ComponentSetValue2( character_platforming_component, "jump_velocity_y", 0)
		elseif(k == "jump_velocity_x") then
			ComponentSetValue2( character_platforming_component, "jump_velocity_x", 0)
		elseif(k == "fly_velocity_x") then
			ComponentSetValue2( character_platforming_component, "fly_velocity_x", 0)
		elseif(k == "fly_speed_mult") then
			ComponentSetValue2( character_platforming_component, "fly_speed_mult", 0)
		elseif(k == "fly_speed_change_spd") then
			ComponentSetValue2( character_platforming_component, "fly_speed_change_spd", 0)
		else
			ComponentSetValue2(character_platforming_component_parent, k, ComponentGetValue2(character_platforming_component, k))
		end
	end
end